@@ -37,10 +37,12 @@
 #include <sys/xattr.h>
 #include "cifsacl.h"
 #include "idmap_plugin.h"
+#include <ftw.h>
 
 static void *plugin_handle;
 static bool plugin_loaded;
 static char *execname;
+static bool raw = false;
 
 static void
 print_each_ace_mask(uint32_t mask)
@@ -330,25 +332,74 @@ getcifsacl_usage(const char *prog)
 	fprintf(stderr,
 	"%s: Display CIFS/NTFS ACL in a security descriptor of a file object\n",
 		prog);
-	fprintf(stderr, "Usage: %s [option] <file_name>\n", prog);
+	fprintf(stderr, "Usage: %s [option] <file_name1> [<file_name2>,<file_name3>,...]\n", prog);
 	fprintf(stderr, "Valid options:\n");
 	fprintf(stderr, "\t-h	Display this help text\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, "\t-v	Version of the program\n");
 	fprintf(stderr, "\n");
+	fprintf(stderr, "\t-R 	recurse into subdirectories\n");
+	fprintf(stderr, "\n");
 	fprintf(stderr, "\t-r	Display raw values of the ACE fields\n");
 	fprintf(stderr, "\nRefer to getcifsacl(1) manpage for details\n");
 }
 
+static void
+getcifsacl(const char *filename)
+{
+	ssize_t attrlen;
+	size_t bufsize = BUFSIZE;
+	char *attrval;
+	int failed = 0;
+cifsacl:
+	if (bufsize >= XATTR_SIZE_MAX) {
+		fprintf(stderr, "buffer to allocate exceeds max size of %d\n",
+			XATTR_SIZE_MAX);
+		exit(1);
+	}
+
+	attrval = malloc(bufsize * sizeof(char));
+	if (!attrval) {
+		fprintf(stderr, "error allocating memory for attribute value buffer\n");
+		exit(1);
+	}
+
+	attrlen = getxattr(filename, ATTRNAME, attrval, bufsize);
+	if (attrlen == -1) {
+		if (errno == ERANGE) {
+			free(attrval);
+			bufsize += BUFSIZE;
+			goto cifsacl;
+		} else {
+			fprintf(stderr, "Failed to getxattr %s: %s\n", filename,
+				strerror(errno));
+			failed = -1;
+		}
+	}
+
+	if (failed == 0) {
+		printf("# filename: %s\n", filename);
+		parse_sec_desc((struct cifs_ntsd *)attrval, attrlen, raw);
+		printf("\n");
+	}
+	free(attrval);
+}
+
+static int recursive(const char *filename, const struct stat *sb, int tflag, struct FTW *ftwbuf)
+{
+	(void)sb;
+	(void)tflag;
+	(void)ftwbuf;
+	getcifsacl(filename);
+	return 0;
+}
+
 int
 main(const int argc, char *const argv[])
 {
 	int c, ret = 0;
-	bool raw = false;
-	ssize_t attrlen;
-	size_t bufsize = BUFSIZE;
-	char *filename, *attrval;
 	execname = basename(argv[0]);
+	int do_recursive = 0;
 
 	if (argc < 2) {
 		fprintf(stderr, "%s: you must specify a filename.\n", execname);
@@ -356,7 +407,7 @@ main(const int argc, char *const argv[])
 		goto out;
 	}
 
-	while ((c = getopt_long(argc, argv, "rhv", NULL, NULL)) != -1) {
+	while ((c = getopt_long(argc, argv, "Rrhv", NULL, NULL)) != -1) {
 		switch (c) {
 		case 'v':
 			printf("Version: %s\n", VERSION);
@@ -364,6 +415,9 @@ main(const int argc, char *const argv[])
 		case 'r':
 			raw = true;
 			break;
+		case 'R':
+			do_recursive = 1;
+			break;
 		default:
 			getcifsacl_usage(execname);
 			goto out;
@@ -372,10 +426,9 @@ main(const int argc, char *const argv[])
 
 	if (optind >= argc) {
 		printf("you must specify a filename after options.\n");
-		printf("Usage: getcifsacl [option] <file_name>\n");
+		printf("Usage: getcifsacl [option] <file_name1> [<file_name2>,<file_name3>,...]\n");
 		goto out;
-	} else
-		filename = argv[optind];
+	}
 
 	if (!raw && !plugin_loaded) {
 		ret = init_plugin(&plugin_handle);
@@ -386,38 +439,15 @@ main(const int argc, char *const argv[])
 			plugin_loaded = true;
 	}
 
-cifsacl:
-	if (bufsize >= XATTR_SIZE_MAX) {
-		printf("buffer to allocate exceeds max size of %d\n",
-				XATTR_SIZE_MAX);
-		ret = -1;
-		goto out;
-	}
-
-	attrval = malloc(bufsize * sizeof(char));
-	if (!attrval) {
-		printf("error allocating memory for attribute value buffer\n");
-		ret = -1;
-		goto out;
-	}
-
-	attrlen = getxattr(filename, ATTRNAME, attrval, bufsize);
-	if (attrlen == -1) {
-		if (errno == ERANGE) {
-			free(attrval);
-			bufsize += BUFSIZE;
-			goto cifsacl;
-		} else {
-			fprintf(stderr, "getxattr failed on %s: %s\n", filename, strerror(errno) );
-			free(attrval);
-			ret = -1;
-			goto out;
+	for(; optind < argc; optind++) {
+		if(do_recursive) {
+			if (nftw(argv[optind], recursive, 20, 0) == -1)
+				fprintf(stderr, "Invalid filename %s: %s\n", argv[optind], strerror(errno));
 		}
+		else
+			getcifsacl(argv[optind]);
 	}
 
-	parse_sec_desc((struct cifs_ntsd *)attrval, attrlen, raw);
-
-	free(attrval);
 out:
 	if (plugin_loaded)
 		exit_plugin(plugin_handle);
